/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * file: ui_X.c *  purpose: handle X graphics 
 */

#include "sins.h"
#include "ui_picts.h"
#include <X11/Xlib.h> 
#include <X11/Xutil.h>   /* XSizeHints */
#include <X11/keysym.h>  /* for a perfect use of keyboard events */

#undef USE_ROOT_WINDOW
#define USE_SIZE_HINTS
#define USE_BACKINGSTORE
#undef DEBUG_X_EVENTS

#define DEFAULTFONTCOLOR "rgb:FFFF/FFFF/0000"
#define DEFAULTFONTDEF "*"
#define MESSAGEHSPACING 5
#define MESSAGEVSPACING 5

/* public functions */
int ui_init ();
int ui_initarena (ARENA *);
void ui_finish (void);
void ui_drawarena (ARENA *);
void ui_drawmessage (char *);
char *ui_prompt (char *);
int ui_getkey (void);
int CMD_loadpictset (char *line);
#ifndef USE_ROOT_WINDOW
int CMD_fontcolor (char *line);
int CMD_loadfont (char *pattern);
#endif

/* private functions */
static void draw_pict (Drawable d, int sx, int sy, int pictnum);
static int *load_pictset (char *filename);
#ifndef USE_ROOT_WINDOW
static Window create_windows (Display *dpy, int width, int height);
static void map_window (Display *dpy, Window w);
#endif
static void create_pictset_pixmap (void);
#ifdef USE_BACKINGSTORE
static void enable_backingstore(Display *d, Window w);
#endif
static int send_pictset (void);
static int copy_pict (int x, int y, int pictnum);
static void redraw_arenarect(int rx, int ry, int rw, int rh);
static void create_graphic_contexts (void);
static void init_colors (void);
#ifndef USE_ROOT_WINDOW
static int load_font (char *);
static int font_color (char *);
#endif

/* private data */
static char *arenamap; /* picture indexes */
static int spriteheight;
static int spritewidth;
static int *pictset;
static int pictsetsent = 0;
static int coloridx[16]; /* index of default 16 colors */

static Display *dpy;
static Screen *scr;
static Window arenawin;
static Pixmap pictset_pixmap;
static GC arenagc; 
static GC messagegc; 
static int ydim, xdim, oldydim, oldxdim;
#ifndef USE_ROOT_WINDOW
static Font font = -1;
static char *fontdef = DEFAULTFONTDEF;
static char *fontcolordef = DEFAULTFONTCOLOR;
static int fontcolor = 0;
static int fontmaxascent;
static int fontmaxdescent;
static int fontminlbearing;
static int fontmaxlbearing;
static int fontminrbearing;
static int fontmaxrbearing;
static char *lastmessage;
#endif


/* public data */
int screen_initialized = 0;

/* Initialize UI module */
int
ui_init ()
{
  if (!register_command ("loadpictset", &CMD_loadpictset,
			 "loadpictset [<fname>]")) return 0;

#ifndef USE_ROOT_WINDOW
  if (!register_command ("fontcolor", &CMD_fontcolor,
			 "fontcolor [<colordef>]")) return 0;

  if (!register_command ("loadfont", &CMD_loadfont,
			 "loadfont [<pattern>]")) return 0;
#endif

  pictset = defpictset;

  return 1;
}

/* initialize the graphics */
int
ui_initarena (ARENA *arena)
{

  /* Connect to the X server */
  if ( ! (dpy=XOpenDisplay(NULL)) ) 
  {
    message ("Can't open display: %s", getenv("DISPLAY"));
    finish();
  }

  scr = DefaultScreenOfDisplay(dpy);

  /* default dimensions */
#ifdef USE_ROOT_WINDOW
  xdim = scr->width;
  ydim = scr->height;
#else
  xdim = (scr->width) * 0.7;
  ydim = (scr->height) * 0.7;
#endif

  /* Set window dimensions based on arena and sprites dimensions */
  if ( !spritewidth ) spritewidth=8;
  if ( !spriteheight ) spriteheight=8;
  if ( arena->lines )
  {
    /* set window height based on arena and sprite heights */
    ydim = spriteheight * ( arena->lines );
  }
  else
  {
    /* set arena height based on window and sprite heights */
    arena->lines = ydim/spriteheight;
    ydim -= ydim%spriteheight;
  }

  if ( arena->cols )
  {
    /* set window width based on arena and sprite widths */
    xdim = spritewidth * ( arena->cols );
  }
  else 
  {
    /* set arena width based on window and sprite widths */
    arena->cols = xdim/spritewidth;
    xdim -= xdim%spritewidth;
  }


  /* remember this original "dimensions" */
  oldxdim = xdim;
  oldydim = ydim;

message("xdim=%d ydim=%d", xdim, ydim);
  create_pictset_pixmap();

#ifdef USE_ROOT_WINDOW
  arenawin = DefaultRootWindow(dpy);
  XSelectInput(dpy, arenawin, ExposureMask);
#else
  arenawin = create_windows(dpy, xdim, ydim);
#endif


  /* We should not rely on this */
#ifdef USE_BACKINGSTORE
  enable_backingstore(dpy, arenawin);
#endif

  create_graphic_contexts();

#ifndef USE_ROOT_WINDOW
  /* load font */
  if ( ! load_font(fontdef) )
  {
    finish();
  }

  /* Set font color */
  if ( ! font_color (fontcolordef) )
  {
    finish();
  }
#endif

  /* Set colors index */
  init_colors();

  /* Would set pictsetsent to 0, so we don't need init_background */
  send_pictset();

  /*
   * cache server's KeySym now ... just to avoid later delay.
   * there should be a better way, but i'm an hacker after all ...
   */
  XKeycodeToKeysym(dpy, 10, 0);

  /* initialize the arena maps (used for optimization purposes) */
  arenamap = (char *)safe_malloc( arena->lines*arena->cols*sizeof(int),
    "ui_X: ui_initarena: arenamap");
  memset(arenamap, -1, arena->lines * arena->cols);

#ifndef USE_ROOT_WINDOW
  /* Map the arena window */
  map_window (dpy, arenawin);
#endif

  /* Set screen initialized flag */
  screen_initialized = 1;

  return 1;

}

/* X version of message drawing function */
void
ui_drawmessage (char *str)
{
#ifndef USE_ROOT_WINDOW
  int mx, my, mw, mh;

  if ( ! str ) return;

  lastmessage = str;

  if ( font == -1 )
  {
#endif
    if ( strlen(str) ) fprintf(stderr, "%s\n", str);
    return;
#ifndef USE_ROOT_WINDOW
  }

  /* Clear previously drawn string */

  /* message box origin */
  mx = MESSAGEHSPACING;
  my = (ARENA_HEIGHT*spriteheight)+MESSAGEVSPACING-1;

  /* message box size */
  mw = xdim;
  mh = ydim-my+1;

  XClearArea(dpy, arenawin, mx, my,
    mw, mh, False);

  XDrawString(dpy, arenawin, messagegc, mx-fontminlbearing,
    my+fontmaxascent, str, strlen(str));
  
  return;
#endif
}

void
ui_drawarena (ARENA *arena)
{
  int y, x;
  sprite *sp;
  int pictidx;
  int should_redraw = 0;
  XEvent event;

#ifndef USE_ROOT_WINDOW
  while ( XCheckWindowEvent(dpy, arenawin, StructureNotifyMask|ExposureMask, &event) )
#else
  while ( XCheckWindowEvent(dpy, arenawin, ExposureMask, &event) )
#endif
  {
    switch(event.type)
    {

#ifndef USE_ROOT_WINDOW
      case (ConfigureNotify):

#ifdef DEBUG_X_EVENTS
        fprintf(stderr, "ui_X.c: Event ConfigureNotify (width=%i, height=%i)\n",
          event.xconfigure.width, event.xconfigure.height);
#endif
        if (event.xconfigure.width != oldxdim || event.xconfigure.height != oldydim)
        {
          oldxdim = event.xconfigure.width;
          oldydim = event.xconfigure.height;
          should_redraw = 1;
        }
        break;
#endif

      case (Expose):
#ifdef DEBUG_X_EVENTS
        fprintf(stderr, "ui_X.c: Event Expose (x=%i, y=%i, w=%i, h=%i)\n", event.xexpose.x,
          event.xexpose.y, event.xexpose.width, event.xexpose.height);
#endif
        /* exposure falls in arena */
        if ( event.xexpose.y < (arena->lines*spriteheight) )
        {
          redraw_arenarect(event.xexpose.x, event.xexpose.y,
            event.xexpose.width, event.xexpose.height);
        }

        /* exposure falls in message box */
        if ( event.xexpose.y+event.xexpose.height >= (arena->lines*spriteheight) )
        {
          ui_drawmessage(lastmessage);
        }

        break;
        
      default:
#ifdef DEBUG_X_EVENTS
        fprintf(stderr, "ui_X.c: Event %i\n", event.type);
#endif
        break;
    }
  }

  /* Send pictset to the display if needed 
   * check it here to allow `loadpictset' command
   * before screen initialization.
   * We need this check also to know wheter to redraw
   * the whole arena ...
   */
  if ( ! pictsetsent )
  {
    send_pictset(); 
    should_redraw = 1;
  }

  for (y = 0; y < arena->lines; y++)
  {
    for (x = 0; x < arena->cols; x++)
    {

      sp = GETSPRITE(arena,y,x);

      if (sp) switch (sp->type)
      {
        case SNAKE:
          pictidx = PICT_SNAKE0+((snakebody *)(sp))->self->playernum;
          break;  
        case EXSNAKE:
          pictidx = PICT_EXSNAKE;
          break; 
        case FRUIT:
          pictidx = PICT_FRUIT0+(((fruit *)(sp))->value)-48;
          break;
        case WALL:
          pictidx = PICT_WALL;
          break;
        default:
          pictidx = PICT_UNKNOWN;
          break;
      }
      else pictidx = PICT_EMPTY;

      if ( *(arenamap+y*arena->cols+x) == pictidx && !should_redraw ) continue;
      *(arenamap+y*arena->cols+x) = pictidx;

      copy_pict(x, y, pictidx);

    }
  }

#ifndef USE_ROOT_WINDOW
  if ( should_redraw ) 
  {
    ui_drawmessage(lastmessage);
  }
#endif

  XSync(dpy, False);
  XFlush(dpy);
}


void
ui_finish ()
{
  if ( screen_initialized )
  {
    XCloseDisplay(dpy);
  }
}


/*
 * Get next key in input buffer
 *
 * OK. We should inspect keyboard status here.
 * It's not that easy. The calling function expect ONE and only ONE char
 * to be returned at a time. It's used to ``consume'' the input buffer
 * before proceding. We need to feed it with all the key pressed up to
 * now I guess... How ?!
 *
 */
int
ui_getkey ()
{
  int retkey = -1;

#ifndef USE_ROOT_WINDOW
  int key = -1;
  XEvent event;
  static int shift = 0;

  while ( XCheckWindowEvent(dpy, arenawin, KeyPressMask|KeyReleaseMask, &event) )
  {

#ifdef DEBUG_X_EVENTS
    switch (event.type)
    {
      case KeyPress:
        fprintf(stderr, "ui_X.c: Event KeyPress (keycode=%i)\n", event.xkey.keycode);
        break;
      case KeyRelease:
        fprintf(stderr, "ui_X.c: Event KeyRelease (keycode=%i)\n", event.xkey.keycode);
        break;
      default:
        fprintf(stderr, "ui_X.c: Event %i\n", event.type);
        break;
    }
#endif

    key = XKeycodeToKeysym(dpy, event.xkey.keycode, shift);
    /* message("ui_getkey: keysym %i / keycode %i", key, event.xkey.keycode); */

    /* handle a shift */
    if ( key == 65505 || key == 65506 )
    {
      shift = event.type == KeyRelease ? 0 : 1;
      continue;
    }

    if ( event.type == KeyPress )
    {

      /* ENTER KEY */
      if ( key == 65293 ) retkey = '\n';
      else if ( key == 65288 ) retkey = 127;

      /* I found this to be working for HIGH-valued keysyms 
       * My nice game segfaults if this function returns numbers
       * higher then 255 (who knows why ? -- me, a char type is expected)
       */
      else if ( key > 256 ) retkey = key % 256;
      else retkey = key;

      break;
    }

  }
#endif

  return retkey;
}

/*
 * prompt the user for a string
 * the line returnered is allocated with malloc(3)
 * so the caller must free it when finished with it. 
 */
char *
ui_prompt (char *prompt_string)
{
  int key;
  int len = 0;
  char *cmdline;

  cmdline = safe_malloc (MAXCMDLINELEN, "prompt input buffer");
  memset(cmdline, '\0', MAXCMDLINELEN);

  message("%s: %s_", prompt_string, cmdline);
  ui_drawarena(&arena);

  while (1)
  {

    ui_drawarena(&arena);

    key=ui_getkey();
    if ( key == -1 ) continue;

    /* max command line lenght reached */
    if ( len > MAXCMDLINELEN ) break;

    if ( key == '\n' ) break;

    /* handle backspace */
    if ( key == 127 )
    {
        if (len) cmdline[--len] = '\0';
    }

    else cmdline[len++] = key;
    message("%s: %s_", prompt_string, cmdline);

  }
  
  ui_drawmessage("");
  return cmdline;
}

int
CMD_loadpictset (char *line)
{
  char fname[MAXPATHLEN];
  char *fpath;
  int ret = 0;
  int *newset;

  if ( line ) ret = sscanf (line, "%s", fname);
  if ( ret != 1 )
  {
    /* set default pictset (should instead complain about wrong num of args?) */
    pictset = defpictset;
    message("default pictset loaded");
    pictsetsent = 0;
    return 1;
  }

  if ( fname[0] != '/' )
  {
    if ( (fpath=find_file(fname)) == NULL )
    {
            message ("Can't find file %s in path %s", fname, path);
            return 0;
    }
  }
  else
  {
    fpath = fname;
  }

  newset = load_pictset(fpath);
  if ( !newset )
  {
    message("Can't load pictset from %s", fpath);
    return 0;
  }

  if ( pictset != defpictset ) free(pictset);
  pictset = newset;

  pictsetsent = 0;

  message("pictset %s loaded", fname);
  return 1;
}

#ifndef USE_ROOT_WINDOW
int
CMD_fontcolor (char *colordef)
{
  if ( ! colordef ) colordef = DEFAULTFONTCOLOR;

  if ( screen_initialized )
  {
    return font_color (colordef);
  }

  fontcolordef = strdup(colordef);
  /* message("fontcolor: No connection to the X-server yet.."); */
  return 0;

}

int
CMD_loadfont (char *pattern)
{

  if ( ! pattern ) fontdef = DEFAULTFONTDEF;
  else fontdef = strdup(pattern);

  if ( screen_initialized )
  {
    return load_font(fontdef);
  }

  return 0;
}
#endif


/************ PRIVATE FUNCTIONS **********************/


#ifdef USE_BACKINGSTORE
static void
enable_backingstore (Display *d, Window w)
{
  XSetWindowAttributes wa;
  wa.backing_store = Always;
  XChangeWindowAttributes(d, w, CWBackingStore, &wa);
}
#endif

#ifndef USE_ROOT_WINDOW
static Window
create_windows (Display *dpy, int width, int height)
{
  Window aw;
#ifdef USE_SIZE_HINTS
  XSizeHints hints;
#endif

  aw = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy), 0, 0,
    width, height, 0, 0, None);

  message("Created arena window %i", aw);

#ifdef USE_SIZE_HINTS
  /* Set Size Hints */
  hints.flags=PSize|PMinSize|PMaxSize;
  hints.min_width=hints.max_width=hints.base_width=width;
  hints.min_height=hints.max_height=hints.base_height=height;
  XSetWMNormalHints(dpy, aw, &hints);
#endif

  /* Give window a name */
  XStoreName (dpy, aw, "X-Sins");

  /* Select events */
  XSelectInput(dpy, aw, KeyPressMask|KeyReleaseMask|StructureNotifyMask|ExposureMask);
  message("Selected events on window %i", aw);

  return aw;
}

static void
map_window (Display *dpy, Window w)
{
  XWindowAttributes wattr;
  XGetWindowAttributes(dpy, w, &wattr);

  /* We want to get MapNotify events */
  XSelectInput(dpy, w, StructureNotifyMask);

  /* "Map" the window (that is, make it appear on the screen) */
  XMapWindow(dpy, w);

  for(;;)
  {
    XEvent e;
    XNextEvent(dpy, &e);
    if (e.type == MapNotify) break;
  }

  XSelectInput(dpy, w, wattr.all_event_masks);
}
#endif


static void 
create_pictset_pixmap (void)
{

  pictset_pixmap = XCreatePixmap (dpy, DefaultRootWindow(dpy), spritewidth,
    spriteheight*PICTSETLEN, DefaultDepthOfScreen(scr));

  message("Created pictset pixmap %i", pictset_pixmap);

}

static void
draw_pict (Drawable w, int sx, int sy, int pictnum)
{
  int x,y,i,j;
  int *pict;
  int color;

  sx=sx*spritewidth;
  sy=sy*spriteheight;
  pict=pictset+pictnum*spritewidth*spriteheight;

  for (x=sx, i=0; i<spritewidth; i++, x++)
  {
    for (y=sy, j=0; j<spriteheight; y++, j++)
    {
      color = *(pict+(j*spritewidth)+i);
      XSetForeground( dpy, arenagc, coloridx[color] );
      XDrawPoint( dpy, w, arenagc, x, y );
    }
  }
}

/*
 * Copy a picture from the pictset pixmap to the specified
 * location on the arenawin
 */
static int
copy_pict (int x, int y, int pictnum)
{
  int sx, sy, dx, dy;

  sx = 0;
  sy = pictnum*spriteheight;
  dx=x*spritewidth;
  dy=y*spriteheight;

  /*
   * This generates NoExpose and GraphicsExpose event 
   * unless the GC has the graphics_exposures set to False
   */
  XCopyArea(dpy,pictset_pixmap,arenawin,arenagc,sx,sy,spritewidth,spriteheight,dx,dy);

  return 1;
}



static int *
load_pictset (char *filename)
{
  FILE *stream;
  int idx = 0;
  int quoted = 0;
  int ch;
  int *set;
  int w, h;

  stream = fopen(filename, "r");
  if ( !stream ) return NULL;

  /* check sprite size */
  if ( fscanf(stream, "#%ix%i\n", &w, &h) != 2 )
  {
    message("Invalid pictset format (missing '#WxH' on line 1)");
    return NULL;
  }

  if ( screen_initialized && (w != spritewidth || h != spriteheight) )
  {
    message("Can't change sprite size while running");
    return NULL;
  }

  spritewidth = w;
  spriteheight = h;
  set = safe_malloc(w*h*PICTSETLEN*sizeof(int), "New pictset");

  while (!feof(stream))
  {
    if ( idx >= w*h*PICTSETLEN ) break;
    ch=getc(stream);
    if (ch) switch(ch)
    {
      case '\n':
      case '\r':
        quoted=0;
        break;
      case ' ':
      case '\t':
        break;
      case '#':
        quoted=1;
        break;
      default:
        if ( !quoted )
        {
          *(set+idx) = ch-97;
          idx++;
        }
        break;
    }
  }
  fclose(stream);
  return set;
}

static int
send_pictset()
{
  int n;

  for (n=0; n<PICTSETLEN; n++)
  {
    draw_pict(pictset_pixmap, 0, n, n);
  }

  message("Pictset sent");
  pictsetsent = 1;

  return 1;
}

static void
redraw_arenarect(int rx, int ry, int rw, int rh)
{
  int ax, ay, aw, ah;
  int y, x;
  int pictidx;

  ax = rx/spritewidth;
  ay = ry/spriteheight;
  aw = rw/spritewidth;
  if ( rw%spritewidth ) aw++;
  ah = rh/spriteheight;
  if ( rh%spriteheight ) ah++;

  for (y = ay; y-ay <= ah && y < ARENA_HEIGHT; y++)
  {
    for (x = ax; x-ax <= aw && x < ARENA_WIDTH; x++)
    {
      pictidx = *(arenamap+y*arena.cols+x);
      copy_pict(x, y, pictidx);
    }
  }

}


static void
create_graphic_contexts()
{
  XGCValues gcval1;

  gcval1.graphics_exposures = False;


  /**************
   * Arena   GC *
   **************/

  /*
   * Create the Graphic Context
   * Set the graphics-exposures to False to avoid
   * NoExpose and GraphicsExpose Event on XCopyArea requests
   *
   * Anyway... NoExpose events suggest there are some problems with XCopyArea
   * ( unreachable source coordinates ? )
   */ 
  arenagc = XCreateGC(dpy, arenawin, GCGraphicsExposures, &gcval1);

  /* Set line width to 0 (an X faq states it might speed up operations ) */
  XSetLineAttributes(dpy, arenagc, 0, LineSolid, CapRound, JoinBevel);

  /**************
   * Message GC *
   **************/

  messagegc = XCreateGC(dpy, arenawin, GCGraphicsExposures, &gcval1);

  /* Set line width to 0 (an X faq states it might speed up operations ) */
  XSetLineAttributes(dpy, messagegc, 0, LineSolid, CapRound, JoinBevel);

}

static void
init_colors()
{
  Colormap cm;
  int i;
  XColor color;
  char *colornames[16] = {
    "rgb:0000/0000/0000", /* black */
    "rgb:0000/0000/AAAA", /* blue */
    "rgb:0000/8888/0000", /* green */
    "rgb:0000/8888/8888", /* cyan */
    "rgb:8888/0000/0000", /* red */
    "rgb:8888/8888/0000", /* magenta */
    "rgb:8888/0000/8888", /* brown */
    "rgb:8888/8888/8888", /* grey */

    "rgb:4444/4444/4444", /* light black */
    "rgb:0000/0000/FFFF", /* light blue */
    "rgb:0000/FFFF/0000", /* light green */
    "rgb:0000/FFFF/FFFF", /* light cyan */
    "rgb:FFFF/0000/0000", /* light red */
    "rgb:FFFF/FFFF/0000", /* light magenta */
    "rgb:FFFF/0000/FFFF", /* light brown */
    "rgb:FFFF/FFFF/FFFF", /* light grey */
  };

  cm = DefaultColormap(dpy, DefaultScreen(dpy));
  /* message("Default colormap = %i", cm); */

  for (i=0; i<16; i++)
  {
    if ( ! XParseColor(dpy, cm, colornames[i], &color) )
    {
      message("color %s not found", colornames[i]);
    }
    XAllocColor(dpy, cm, &color);
    coloridx[i] = color.pixel;
  }

}


#ifndef USE_ROOT_WINDOW
/*
 * Load font
 */
static int
load_font (char *pattern)
{
#ifdef USE_SIZE_HINTS
  XSizeHints hints;
#endif
  XWindowChanges wc;
  int new_arenawin_height;
  XFontStruct *fontinfo;
  char **fontnames;
  int foundfonts = 0;

  if ( ! pattern ) pattern = DEFAULTFONTDEF;

  if ( ! (fontnames=XListFontsWithInfo(dpy, pattern, 1, &foundfonts, &fontinfo)) || ! foundfonts )
  {
    message("No fonts matching pattern %s", pattern);
    return 0;
  }

  /* Unload old font if any */
  if ( font != -1 ) XUnloadFont(dpy, font);

  /* Load the font */
  font = XLoadFont(dpy, fontnames[0]);

  /* Announce the font load */
  message("Loaded font %s", fontnames[0]);

  /* Remember font definition (useless) */
  fontdef = pattern;

  /* Write down font dimensions */
  fontmaxascent = fontinfo->max_bounds.ascent;
  fontmaxdescent = fontinfo->max_bounds.descent;
  fontminrbearing = fontinfo->min_bounds.rbearing;
  fontmaxrbearing = fontinfo->max_bounds.rbearing;
  fontminlbearing = fontinfo->min_bounds.lbearing;
  fontmaxlbearing = fontinfo->max_bounds.lbearing;

  /* Resize arena window */
  new_arenawin_height = ARENA_HEIGHT*spriteheight +
    MESSAGEVSPACING*2 + fontmaxascent + fontmaxdescent;
  wc.height = new_arenawin_height;
  XConfigureWindow(dpy, arenawin, CWHeight, &wc);

#ifdef USE_SIZE_HINTS
  /* Set Size Hints */
  hints.flags=PSize|PMinSize|PMaxSize;
  hints.min_height=hints.max_height=hints.base_height=new_arenawin_height;
  hints.min_width=hints.max_width=hints.base_width=ARENA_WIDTH*spritewidth;
  XSetWMNormalHints(dpy,arenawin,&hints);
#endif


  XFreeFontInfo (fontnames, fontinfo, foundfonts);

  XSetFont(dpy, messagegc, font);

  return 1;
}

/*
 * Font color
 */
static int
font_color (char *def)
{
  XColor color;
  Colormap cm;

  if ( ! def ) def = DEFAULTFONTCOLOR;

  cm = DefaultColormap(dpy, DefaultScreen(dpy));
  if ( ! XParseColor(dpy, cm, def, &color) )
  {
      message("color %s not found", def);
      return 0;
  }
  XAllocColor(dpy, cm, &color);
  fontcolor = color.pixel;
  XSetForeground(dpy, messagegc, fontcolor);
  fontcolordef = def;
  message("Font color changed to %s in messagegc", def);
  return 1;
}
#endif

